@@ -1,4 +1,4 @@
-#  Copyright (c) 2022 Mira Geoscience Ltd.
+#  Copyright (c) 2023 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -15,6 +15,8 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with geoh5py.  If not, see <https://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 import uuid
 
 from .data import Data
@@ -29,7 +31,7 @@ class UnknownData(Data):
         data_type: DataType,
         association: DataAssociationEnum,
         name: str,
-        uid: uuid.UUID = None,
+        uid: uuid.UUID | None = None,
     ):
         super().__init__(data_type, association=association, name=name, uid=uid)
 
