@@ -1,4 +1,4 @@
-#  Copyright (c) 2022 Mira Geoscience Ltd.
+#  Copyright (c) 2023 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -18,7 +18,6 @@
 from __future__ import annotations
 
 import uuid
-import warnings
 
 from geoh5py.objects.curve import Curve
 from geoh5py.objects.object_type import ObjectType
@@ -70,17 +69,13 @@ class BaseTipper(BaseEMSurvey):
 
                 if isinstance(base_station_entity, TipperBaseStations):
                     self._base_stations = base_station_entity
-                else:
-                    warnings.warn(
-                        "Associated `base_stations` entity not set.", UserWarning
-                    )
 
         return self._base_stations
 
     @base_stations.setter
     def base_stations(self, base: TipperBaseStations):
         if not isinstance(base, (TipperBaseStations, type(None))):
-            raise AttributeError(
+            raise TypeError(
                 f"Input `base_stations` must be of type '{TipperBaseStations}' or None"
             )
 
@@ -99,9 +94,23 @@ class BaseTipper(BaseEMSurvey):
 
     @property
     def default_input_types(self) -> list[str]:
-        """Input types. Must be 'Rx and base stations'"""
+        """Choice of survey creation types."""
         return self.__INPUT_TYPE
 
+    @property
+    def default_receiver_type(self):
+        """
+        :return: Transmitter class
+        """
+        return TipperReceivers
+
+    @property
+    def default_transmitter_type(self):
+        """
+        :return: Transmitter class
+        """
+        return type(None)
+
     @property
     def default_metadata(self) -> dict:
         """
@@ -135,10 +144,10 @@ class TipperReceivers(BaseTipper, Curve):  # pylint: disable=too-many-ancestors
     __TYPE_UID = uuid.UUID("{0b639533-f35b-44d8-92a8-f70ecff3fd26}")
     __TYPE = "Receivers"
 
-    def __init__(self, object_type: ObjectType, **kwargs):
+    def __init__(self, object_type: ObjectType, name="Tipper rx", **kwargs):
         self._base_stations: TipperBaseStations | None = None
 
-        super().__init__(object_type, **kwargs)
+        super().__init__(object_type, name=name, **kwargs)
 
     @classmethod
     def default_type_uid(cls) -> uuid.UUID:
@@ -161,9 +170,9 @@ class TipperBaseStations(BaseTipper, Points):
     __TYPE_UID = uuid.UUID("{f495cd13-f09b-4a97-9212-2ea392aeb375}")
     __TYPE = "Base stations"
 
-    def __init__(self, object_type: ObjectType, **kwargs):
+    def __init__(self, object_type: ObjectType, name="Tipper base", **kwargs):
 
-        super().__init__(object_type, **kwargs)
+        super().__init__(object_type, name=name, **kwargs)
 
     @classmethod
     def default_type_uid(cls) -> uuid.UUID:
@@ -172,13 +181,6 @@ class TipperBaseStations(BaseTipper, Points):
         """
         return cls.__TYPE_UID
 
-    @property
-    def default_receiver_type(self):
-        """
-        :return: Receiver class
-        """
-        return TipperReceivers
-
     @property
     def type(self):
         """Survey element type"""
