@@ -1,4 +1,4 @@
-#  Copyright (c) 2022 Mira Geoscience Ltd.
+#  Copyright (c) 2023 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -367,3 +367,15 @@ def get_attributes(entity, omit_list=(), attributes=None):
 
             attributes[key] = getattr(entity, key)
     return attributes
+
+
+def overwrite_kwargs(to_overwrite: dict, kwargs_to_add: dict) -> dict:
+    """
+    Overwrite kwargs with overwrite.
+    :param to_overwrite: Dictionary of kwargs to overwrite.
+    :param kwargs_to_add: Dictionary of kwargs to modify to_overwrite.
+    """
+    for key, value in kwargs_to_add.items():
+        if key in to_overwrite:
+            to_overwrite[key] = value
+    return to_overwrite
