@@ -1,4 +1,4 @@
-#  Copyright (c) 2022 Mira Geoscience Ltd.
+#  Copyright (c) 2023 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -61,7 +61,12 @@ from ..shared.concatenation import (
 )
 from ..shared.entity import Entity
 from ..shared.exceptions import Geoh5FileClosedError
-from ..shared.utils import as_str_if_utf8_bytes, get_attributes, str2uuid
+from ..shared.utils import (
+    as_str_if_utf8_bytes,
+    get_attributes,
+    overwrite_kwargs,
+    str2uuid,
+)
 
 if TYPE_CHECKING:
     from ..groups import group
@@ -79,7 +84,7 @@ class Workspace(AbstractContextManager):
         A new project is created if the target file cannot by found on disk.
     """
 
-    _active_ref: ClassVar[ReferenceType[Workspace]] = type(None)  # type: ignore
+    _active_ref: ClassVar[ReferenceType[Workspace]] | type(None) = type(None)  # type: ignore
 
     _attribute_map = {
         "Contributors": "contributors",
@@ -223,6 +228,7 @@ class Workspace(AbstractContextManager):
         copy_children: bool = True,
         omit_list: tuple = (),
         extent: np.ndarray | None = None,
+        **kwargs,
     ):
         """
         Copy an entity to a different parent with copies of children.
@@ -231,7 +237,8 @@ class Workspace(AbstractContextManager):
         :param parent: Target parent to copy the entity under.
         :param copy_children: Copy all children of the entity.
         :param omit_list: List of property names to omit on copy
-        :param mask: Clip object's copy by extent defined by a South-West and North-East corners.
+        :param extent: Clip object's copy by extent defined by a South-West and North-East corners.
+        :param kwargs: Additional keyword arguments passed to the copy constructor.
 
         :return: The Entity registered to the workspace.
         """
@@ -249,6 +256,10 @@ class Workspace(AbstractContextManager):
             omit_list=["_workspace", "_on_file"] + list(omit_list),
         )
 
+        # overwrite kwargs
+        entity_kwargs = overwrite_kwargs(entity_kwargs, kwargs)
+        entity_type_kwargs = overwrite_kwargs(entity_type_kwargs, kwargs)
+
         if not isinstance(parent, (ObjectBase, Group, Workspace)):
             raise ValueError(
                 "Input 'parent' should be of type (ObjectBase, Group, Workspace)"
@@ -308,20 +319,31 @@ class Workspace(AbstractContextManager):
             new_group.properties = [data_map[uid] for uid in prop_group.properties]
 
     def create_from_concatenation(self, attributes):
-        if "Object Type ID" in attributes:
-            class_type = ObjectBase
-            type_attr = {"uid": attributes["Object Type ID"]}
-        else:
-            class_type = Data
-            type_attr = self.fetch_type(uuid.UUID(attributes["Type ID"]), "Data")
-
         if "Name" in attributes:
             attributes["Name"] = attributes["Name"].replace("\u2044", "/")
-        recovered_entity = self.create_entity(
-            class_type,
-            save_on_creation=False,
-            **{"entity": attributes, "entity_type": type_attr},
-        )
+
+        if "Object Type ID" in attributes:
+            recovered_entity = self.create_entity(
+                ObjectBase,
+                save_on_creation=False,
+                **{
+                    "entity": attributes,
+                    "entity_type": {"uid": attributes["Object Type ID"]},
+                },
+            )
+
+        else:
+            recovered_entity = self.create_entity(
+                Data,
+                save_on_creation=False,
+                **{
+                    "entity": attributes,
+                    "entity_type": self.fetch_type(
+                        uuid.UUID(attributes["Type ID"]), "Data"
+                    ),
+                },
+            )
+
         if recovered_entity is not None:
             recovered_entity.on_file = True
             recovered_entity.entity_type.on_file = True
@@ -486,13 +508,14 @@ class Workspace(AbstractContextManager):
         return self._all_data()
 
     def fetch_or_create_root(self):
-        try:
-            self._root = self.load_entity(uuid.uuid4(), "root")
+        root = self.load_entity(uuid.uuid4(), "root")
+        if root:
+            self._root = root
             self._root.on_file = True
             self._root.entity_type.on_file = True
             self.fetch_children(self._root, recursively=True)
 
-        except KeyError:
+        else:
             self._root = self.create_entity(RootGroup, save_on_creation=False)
 
             for entity_type in ["group", "object"]:
@@ -678,11 +701,13 @@ class Workspace(AbstractContextManager):
 
         return family_tree
 
-    def fetch_concatenated_attributes(self, entity: Group | ObjectBase) -> dict | None:
+    def fetch_concatenated_attributes(
+        self, entity: Concatenator | ConcatenatedObject
+    ) -> dict | None:
         """
-        Fetch attributes of ConcatenatedData entities.
+        Fetch attributes of Concatenated entity.
 
-        :param entity: Concatenator group.
+        :param entity: Concatenator group or ConcatenateObject.
 
         :return: Dictionary of attributes.
         """
@@ -698,7 +723,7 @@ class Workspace(AbstractContextManager):
             H5Reader.fetch_concatenated_attributes,
             entity.uid,
             entity_type,
-            "Attributes",
+            entity.concat_attr_str,
             mode="r",
         )
 
@@ -992,7 +1017,7 @@ class Workspace(AbstractContextManager):
         self,
         uid: uuid.UUID,
         entity_type: str,
-        parent: Entity = None,
+        parent: Entity | None = None,
     ) -> Entity | None:
         """
         Recover an entity from geoh5.
@@ -1011,23 +1036,24 @@ class Workspace(AbstractContextManager):
             "data": Data,
             "root": RootGroup,
         }
-        (
-            attributes,
-            type_attributes,
-            property_groups,
-        ) = self._io_call(H5Reader.fetch_attributes, uid, entity_type, mode="r")
+        attributes = self._io_call(
+            H5Reader.fetch_attributes, uid, entity_type, mode="r"
+        )
+
+        if attributes is None:
+            return None
 
         if parent is not None:
-            attributes["entity"]["parent"] = parent
+            attributes[0]["entity"]["parent"] = parent
 
         entity = self.create_entity(
             base_classes[entity_type],
             save_on_creation=False,
-            **{**attributes, **type_attributes},
+            **{**attributes[0], **attributes[1]},
         )
 
-        if isinstance(entity, ObjectBase) and len(property_groups) > 0:
-            for kwargs in property_groups.values():
+        if isinstance(entity, ObjectBase) and len(attributes[2]) > 0:
+            for kwargs in attributes[2].values():
                 entity.find_or_create_property_group(**kwargs)
 
         return entity
@@ -1162,7 +1188,11 @@ class Workspace(AbstractContextManager):
         return self._all_types()
 
     def update_attribute(
-        self, entity: Entity | EntityType, attribute: str, channel: str = None, **kwargs
+        self,
+        entity: Entity | EntityType,
+        attribute: str,
+        channel: str | None = None,
+        **kwargs,
     ) -> None:
         """
         Save or update an entity to geoh5.
